@@ -16,12 +16,15 @@
 
 #include "kformat.h"
 
+using namespace Qt::Literals;
+
 void setupEnvironment()
 {
 #ifndef Q_OS_WIN
     // ignore translations
     qputenv("XDG_DATA_DIRS", "does-not-exist");
 #endif
+    qputenv("TZ", "Asia/Kolkata");
 }
 Q_CONSTRUCTOR_FUNCTION(setupEnvironment)
 
@@ -274,6 +277,25 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("14708m03.700s"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("179m59.900s"));
     QCOMPARE(format.formatDuration(largeValue, options), QStringLiteral("166666m39.999s"));
+
+    // ensure inline options work
+    QCOMPARE(format.formatDuration(12 * MSecsInHour + 59 * MSecsInMinute, KFormat::AbbreviatedDuration | KFormat::HideSeconds), "12 hr 59 min"_L1);
+}
+
+void KFormatTest::formatAbbreviatedDuration()
+{
+    KFormat fmt(QLocale::c());
+    KFormat::DurationFormatOptions opt = KFormat::AbbreviatedDuration;
+
+    QCOMPARE(fmt.formatDuration(0, opt), "0 sec"_L1);
+    QCOMPARE(fmt.formatDuration(0, opt | KFormat::HideSeconds), "0 min"_L1);
+    QCOMPARE(fmt.formatDuration(MSecsInSecond, opt), "1 sec"_L1);
+    QCOMPARE(fmt.formatDuration(MSecsInMinute, opt), "1 min 0 sec"_L1);
+    QCOMPARE(fmt.formatDuration(MSecsInMinute, opt | KFormat::HideSeconds), "1 min"_L1);
+    QCOMPARE(fmt.formatDuration(MSecsInHour, opt), "1 hr 0 min 0 sec"_L1);
+    QCOMPARE(fmt.formatDuration(MSecsInHour, opt | KFormat::HideSeconds), "1 hr 0 min"_L1);
+    QCOMPARE(fmt.formatDuration(MSecsInHour, opt | KFormat::HideSeconds | KFormat::FoldHours), "60 min"_L1);
+    QCOMPARE(fmt.formatDuration(12 * MSecsInHour + 59 * MSecsInMinute, opt | KFormat::HideSeconds), "12 hr 59 min"_L1);
 }
 
 void KFormatTest::formatDecimalDuration()
@@ -374,10 +396,18 @@ void KFormatTest::formatRelativeDate()
     testDateTime = now.addSecs(-1);
     QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), QStringLiteral("Just now"));
 
+    // 1 second ahead
+    testDateTime = now.addSecs(1);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), QStringLiteral("Now"));
+
     // 5 minutes ago
     testDateTime = now.addSecs(-300);
     QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), QStringLiteral("5 minute(s) ago"));
 
+    // 5 minutes ahead
+    testDateTime = now.addSecs(300);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), "In 5 minute(s)"_L1);
+
     testDateTime = QDateTime(QDate::currentDate().addDays(8), QTime(3, 0, 0));
     QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::LongFormat),
              QStringLiteral("%1 at %2")
@@ -396,6 +426,82 @@ void KFormatTest::formatRelativeDate()
     QLocale frenchLocal = QLocale::French;
     KFormat formatFrench(frenchLocal);
     QCOMPARE(formatFrench.formatRelativeDateTime(testDateTime, QLocale::LongFormat), QStringLiteral("Dimanche 3 octobre 2021 at 05:33"));
+
+    // narrow format
+    now = QDateTime::currentDateTime();
+    testDateTime = now.addSecs(-1);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "Just now"_L1);
+    testDateTime = now.addSecs(1);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "Now"_L1);
+
+    testDateTime = now.addSecs(-300);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "5 min(s) ago"_L1);
+    testDateTime = now.addSecs(300);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "5 min(s)"_L1);
+}
+
+void KFormatTest::formatTime()
+{
+    KFormat fmt(QLocale(u"en_GB"));
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone::LocalTime}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation), "12:23"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone::utc()}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation), "12:23 UTC"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone::utc()}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded), "12:23 UTC"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Asia/Kolkata")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Asia/Kolkata")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation), "12:23 IST"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 CEST"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation),
+             "12:23 CET"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::DoNotAddTimeZone), "12:23"_L1);
+
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone::fromSecondsAheadOfUtc(0)}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 UTC"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone::fromSecondsAheadOfUtc(8 * 3600)},
+                            QLocale::ShortFormat,
+                            KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 UTC+08:00"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone::fromSecondsAheadOfUtc(5.5 * 3600)},
+                            QLocale::ShortFormat,
+                            KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23"_L1);
+
+    KFormat de_fmt(QLocale(u"de_DE"));
+    QCOMPARE(de_fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 MESZ"_L1);
+}
+
+void KFormatTest::formatDistance()
+{
+    {
+        KFormat fmt(QLocale(u"en_US"));
+        QCOMPARE(fmt.formatDistance(0.0), "0 ft"_L1);
+        QCOMPARE(fmt.formatDistance(0.1), "0 ft"_L1);
+        QCOMPARE(fmt.formatDistance(0.3), "1 ft"_L1);
+        QCOMPARE(fmt.formatDistance(152.3), "500 ft"_L1);
+        QCOMPARE(fmt.formatDistance(152.5), "0.1 mi"_L1);
+        QCOMPARE(fmt.formatDistance(2414.6), "1.5 mi"_L1);
+        QCOMPARE(fmt.formatDistance(16100.0), "10 mi"_L1);
+        QCOMPARE(fmt.formatDistance(1610000.0), "1,000 mi"_L1);
+
+        QCOMPARE(fmt.formatDistance(0.0, KFormat::MetricDistanceUnits), "0 m"_L1);
+        QCOMPARE(fmt.formatDistance(0.1, KFormat::MetricDistanceUnits), "0 m"_L1);
+        QCOMPARE(fmt.formatDistance(0.5, KFormat::MetricDistanceUnits), "1 m"_L1);
+        QCOMPARE(fmt.formatDistance(123.4, KFormat::MetricDistanceUnits), "123 m"_L1);
+        QCOMPARE(fmt.formatDistance(1234.5, KFormat::MetricDistanceUnits), "1.2 km"_L1);
+        QCOMPARE(fmt.formatDistance(12345.6, KFormat::MetricDistanceUnits), "12 km"_L1);
+        QCOMPARE(fmt.formatDistance(1234567.8, KFormat::MetricDistanceUnits), "1,235 km"_L1);
+    }
+    {
+        KFormat fmt(QLocale(u"de_DE"));
+        QCOMPARE(fmt.formatDistance(0.0), "0 m"_L1);
+        QCOMPARE(fmt.formatDistance(0.1), "0 m"_L1);
+        QCOMPARE(fmt.formatDistance(0.5), "1 m"_L1);
+        QCOMPARE(fmt.formatDistance(123.4), "123 m"_L1);
+        QCOMPARE(fmt.formatDistance(1234.5), "1,2 km"_L1);
+        QCOMPARE(fmt.formatDistance(12345.6), "12 km"_L1);
+        QCOMPARE(fmt.formatDistance(1234567.8, KFormat::MetricDistanceUnits), "1.235 km"_L1);
+    }
 }
 
 QTEST_MAIN(KFormatTest)
